@@ -270,9 +270,6 @@ func (env *environment) getRuntimeGOOS() string {
 }
 
 func (env *environment) runCommand(command string, args ...string) (string, error) {
-	if env.getRuntimeGOOS() == windowsPlatform && !strings.HasSuffix(command, ".exe") {
-		command += ".exe"
-	}
 	defer env.trace(time.Now(), "runCommand", append([]string{command}, args...)...)
 	if cmd, ok := env.cmdCache.get(command); ok {
 		command = cmd
@@ -402,15 +399,42 @@ func (env *environment) getBatteryInfo() ([]*battery.Battery, error) {
 			validBatteries = append(validBatteries, batt)
 		}
 	}
+	// clean minor errors
 	unableToRetrieveBatteryInfo := "A device which does not exist was specified."
+	unknownChargeRate := "Unknown value received"
+	var fatalErr battery.Errors
+	ignoreErr := func(err error) bool {
+		if e, ok := err.(battery.ErrPartial); ok {
+			// ignore unknown charge rate value error
+			if e.Current == nil &&
+				e.Design == nil &&
+				e.DesignVoltage == nil &&
+				e.Full == nil &&
+				e.State == nil &&
+				e.Voltage == nil &&
+				e.ChargeRate != nil &&
+				e.ChargeRate.Error() == unknownChargeRate {
+				return true
+			}
+		}
+		return false
+	}
+	if batErr, ok := err.(battery.Errors); ok {
+		for _, err := range batErr {
+			if !ignoreErr(err) {
+				fatalErr = append(fatalErr, err)
+			}
+		}
+	}
+
 	// when battery info fails to get retrieved but there is at least one valid battery, return it without error
-	if len(validBatteries) > 0 && err != nil && strings.Contains(err.Error(), unableToRetrieveBatteryInfo) {
+	if len(validBatteries) > 0 && fatalErr != nil && strings.Contains(fatalErr.Error(), unableToRetrieveBatteryInfo) {
 		return validBatteries, nil
 	}
 	// another error occurred (possibly unmapped use-case), return it
-	if err != nil {
-		env.log(Error, "getBatteryInfo", err.Error())
-		return nil, err
+	if fatalErr != nil {
+		env.log(Error, "getBatteryInfo", fatalErr.Error())
+		return nil, fatalErr
 	}
 	// everything is fine
 	return validBatteries, nil
