@@ -13,8 +13,7 @@ import (
 func TestGetStatusDetailStringDefault(t *testing.T) {
 	expected := "icon +1"
 	status := &GitStatus{
-		Changed: true,
-		Added:   1,
+		Added: 1,
 	}
 	g := &git{}
 	assert.Equal(t, expected, g.getStatusDetailString(status, WorkingColor, LocalWorkingIcon, "icon"))
@@ -23,8 +22,7 @@ func TestGetStatusDetailStringDefault(t *testing.T) {
 func TestGetStatusDetailStringDefaultColorOverride(t *testing.T) {
 	expected := "<#123456>icon +1</>"
 	status := &GitStatus{
-		Changed: true,
-		Added:   1,
+		Added: 1,
 	}
 	var props properties = map[Property]interface{}{
 		WorkingColor: "#123456",
@@ -38,8 +36,7 @@ func TestGetStatusDetailStringDefaultColorOverride(t *testing.T) {
 func TestGetStatusDetailStringDefaultColorOverrideAndIconColorOverride(t *testing.T) {
 	expected := "<#789123>work</> <#123456>+1</>"
 	status := &GitStatus{
-		Changed: true,
-		Added:   1,
+		Added: 1,
 	}
 	var props properties = map[Property]interface{}{
 		WorkingColor:     "#123456",
@@ -54,8 +51,7 @@ func TestGetStatusDetailStringDefaultColorOverrideAndIconColorOverride(t *testin
 func TestGetStatusDetailStringDefaultColorOverrideNoIconColorOverride(t *testing.T) {
 	expected := "<#123456>work +1</>"
 	status := &GitStatus{
-		Changed: true,
-		Added:   1,
+		Added: 1,
 	}
 	var props properties = map[Property]interface{}{
 		WorkingColor:     "#123456",
@@ -70,8 +66,7 @@ func TestGetStatusDetailStringDefaultColorOverrideNoIconColorOverride(t *testing
 func TestGetStatusDetailStringNoStatus(t *testing.T) {
 	expected := "icon"
 	status := &GitStatus{
-		Changed: true,
-		Added:   1,
+		Added: 1,
 	}
 	var props properties = map[Property]interface{}{
 		DisplayStatusDetail: false,
@@ -85,8 +80,7 @@ func TestGetStatusDetailStringNoStatus(t *testing.T) {
 func TestGetStatusDetailStringNoStatusColorOverride(t *testing.T) {
 	expected := "<#123456>icon</>"
 	status := &GitStatus{
-		Changed: true,
-		Added:   1,
+		Added: 1,
 	}
 	var props properties = map[Property]interface{}{
 		DisplayStatusDetail: false,
@@ -106,8 +100,9 @@ func TestGetStatusColorLocalChangesStaging(t *testing.T) {
 	g := &git{
 		props: props,
 		Staging: &GitStatus{
-			Changed: true,
+			Modified: 1,
 		},
+		Working: &GitStatus{},
 	}
 	assert.Equal(t, expected, g.getStatusColor("#fg1111"))
 }
@@ -121,7 +116,7 @@ func TestGetStatusColorLocalChangesWorking(t *testing.T) {
 		props:   props,
 		Staging: &GitStatus{},
 		Working: &GitStatus{
-			Changed: true,
+			Modified: 1,
 		},
 	}
 	assert.Equal(t, expected, g.getStatusColor("#fg1111"))
@@ -196,8 +191,9 @@ func TestSetStatusColorForeground(t *testing.T) {
 	g := &git{
 		props: props,
 		Staging: &GitStatus{
-			Changed: true,
+			Added: 1,
 		},
+		Working: &GitStatus{},
 	}
 	g.SetStatusColor()
 	assert.Equal(t, expected, g.props[ForegroundOverride])
@@ -210,9 +206,10 @@ func TestSetStatusColorBackground(t *testing.T) {
 		ColorBackground:   true,
 	}
 	g := &git{
-		props: props,
-		Staging: &GitStatus{
-			Changed: true,
+		props:   props,
+		Staging: &GitStatus{},
+		Working: &GitStatus{
+			Modified: 1,
 		},
 	}
 	g.SetStatusColor()
@@ -221,16 +218,32 @@ func TestSetStatusColorBackground(t *testing.T) {
 
 func TestStatusColorsWithoutDisplayStatus(t *testing.T) {
 	expected := changesColor
-	context := &detachedContext{
-		status: "## main...origin/main [ahead 33]\n M myfile",
+	status := "## main...origin/main [ahead 33]\n M myfile"
+	env := new(MockedEnvironment)
+	env.On("isWsl", nil).Return(false)
+	env.On("getRuntimeGOOS", nil).Return("unix")
+	env.On("hasFolder", "/rebase-merge").Return(false)
+	env.On("hasFolder", "/rebase-apply").Return(false)
+	env.On("hasFolder", "/sequencer").Return(false)
+	env.On("getFileContent", "/HEAD").Return(status)
+	env.On("hasFilesInDir", "", "CHERRY_PICK_HEAD").Return(false)
+	env.On("hasFilesInDir", "", "REVERT_HEAD").Return(false)
+	env.On("hasFilesInDir", "", "MERGE_MSG").Return(false)
+	env.On("hasFilesInDir", "", "MERGE_HEAD").Return(false)
+	env.On("hasFilesInDir", "", "sequencer/todo").Return(false)
+	env.mockGitCommand("", "describe", "--tags", "--exact-match")
+	env.mockGitCommand(status, "status", "-unormal", "--branch", "--porcelain=2")
+	g := &git{
+		env:              env,
+		gitWorkingFolder: "",
+		props: map[Property]interface{}{
+			DisplayStatus:       false,
+			StatusColorsEnabled: true,
+			LocalChangesColor:   expected,
+		},
 	}
-	g := setupHEADContextEnv(context)
-	var props properties = map[Property]interface{}{
-		DisplayStatus:       false,
-		StatusColorsEnabled: true,
-		LocalChangesColor:   expected,
-	}
-	g.props = props
+	g.Working = &GitStatus{}
+	g.Staging = &GitStatus{}
 	g.string()
 	assert.Equal(t, expected, g.props[BackgroundOverride])
 }
@@ -596,3 +609,108 @@ func TestPropertySessionSegment(t *testing.T) {
 		}
 	}
 }
+
+// Language
+
+func TestLanguageVersionMismatch(t *testing.T) {
+	cases := []struct {
+		Case            string
+		Enabled         bool
+		Mismatch        bool
+		ExpectedColor   string
+		ColorBackground bool
+	}{
+		{Case: "Mismatch - Foreground color", Enabled: true, Mismatch: true, ExpectedColor: "#566777"},
+		{Case: "Mismatch - Background color", Enabled: true, Mismatch: true, ExpectedColor: "#566777", ColorBackground: true},
+		{Case: "Disabled", Enabled: false},
+		{Case: "No mismatch", Enabled: true, Mismatch: false},
+	}
+	for _, tc := range cases {
+		props := map[Property]interface{}{
+			EnableVersionMismatch: tc.Enabled,
+			VersionMismatchColor:  tc.ExpectedColor,
+			ColorBackground:       tc.ColorBackground,
+		}
+		var matchesVersionFile func() bool
+		switch tc.Mismatch {
+		case true:
+			matchesVersionFile = func() bool {
+				return false
+			}
+		default:
+			matchesVersionFile = func() bool {
+				return true
+			}
+		}
+		args := &languageArgs{
+			commands: []*cmd{
+				{
+					executable: "unicorn",
+					args:       []string{"--version"},
+					regex:      "(?P<version>.*)",
+				},
+			},
+			extensions:         []string{uni, corn},
+			enabledExtensions:  []string{uni, corn},
+			enabledCommands:    []string{"unicorn"},
+			version:            universion,
+			properties:         props,
+			matchesVersionFile: matchesVersionFile,
+		}
+		lang := bootStrapLanguageTest(args)
+		assert.True(t, lang.enabled(), tc.Case)
+		assert.Equal(t, universion, lang.string(), tc.Case)
+		if tc.ColorBackground {
+			assert.Equal(t, tc.ExpectedColor, lang.props[BackgroundOverride], tc.Case)
+			return
+		}
+		assert.Equal(t, tc.ExpectedColor, lang.props.getColor(ForegroundOverride, ""), tc.Case)
+	}
+}
+
+// Python
+
+func TestPythonVirtualEnv(t *testing.T) {
+	cases := []struct {
+		Case                string
+		Expected            string
+		ExpectedDisabled    bool
+		VirtualEnvName      string
+		CondaEnvName        string
+		CondaDefaultEnvName string
+		PyEnvName           string
+		FetchVersion        bool
+		DisplayDefault      bool
+	}{
+		{Case: "VENV", Expected: "VENV", VirtualEnvName: "VENV"},
+		{Case: "CONDA", Expected: "CONDA", CondaEnvName: "CONDA"},
+		{Case: "CONDA default", Expected: "CONDA", CondaDefaultEnvName: "CONDA"},
+		{Case: "Display Base", Expected: "base", CondaDefaultEnvName: "base", DisplayDefault: true},
+		{Case: "Hide base", Expected: "", CondaDefaultEnvName: "base", ExpectedDisabled: true},
+		{Case: "PYENV", Expected: "PYENV", PyEnvName: "PYENV"},
+		{Case: "PYENV Version", Expected: "PYENV 3.8.4", PyEnvName: "PYENV", FetchVersion: true},
+	}
+
+	for _, tc := range cases {
+		env := new(MockedEnvironment)
+		env.On("hasCommand", "python").Return(true)
+		env.On("runCommand", "python", []string{"--version"}).Return("Python 3.8.4", nil)
+		env.On("hasFiles", "*.py").Return(true)
+		env.On("getenv", "VIRTUAL_ENV").Return(tc.VirtualEnvName)
+		env.On("getenv", "CONDA_ENV_PATH").Return(tc.CondaEnvName)
+		env.On("getenv", "CONDA_DEFAULT_ENV").Return(tc.CondaDefaultEnvName)
+		env.On("getenv", "PYENV_VERSION").Return(tc.PyEnvName)
+		env.On("getPathSeperator", nil).Return("")
+		env.On("getcwd", nil).Return("/usr/home/project")
+		env.On("homeDir", nil).Return("/usr/home")
+		var props properties = map[Property]interface{}{
+			FetchVersion:      tc.FetchVersion,
+			DisplayVirtualEnv: true,
+			DisplayDefault:    tc.DisplayDefault,
+		}
+		python := &python{}
+		python.init(props, env)
+		assert.Equal(t, !tc.ExpectedDisabled, python.enabled(), tc.Case)
+		assert.Equal(t, tc.Expected, python.string(), tc.Case)
+	}
+}
