@@ -2,6 +2,7 @@ package main
 
 import (
 	"fmt"
+	"strings"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
@@ -9,6 +10,7 @@ import (
 
 const (
 	changesColor = "#BD8BDE"
+	branchName   = "main"
 )
 
 func TestEnabledGitNotFound(t *testing.T) {
@@ -76,299 +78,301 @@ func TestGetGitOutputForCommand(t *testing.T) {
 	assert.Equal(t, want, got)
 }
 
-type detachedContext struct {
-	currentCommit string
-	rebase        string
-	rebaseMerge   bool
-	rebaseApply   bool
-	origin        string
-	onto          string
-	step          string
-	total         string
-	branchName    string
-	tagName       string
-	cherryPick    bool
-	cherryPickSHA string
-	revert        bool
-	revertSHA     string
-	sequencer     bool
-	sequencerTodo string
-	merge         bool
-	mergeHEAD     string
-	mergeMsgStart string
-	status        string
-}
-
-func setupHEADContextEnv(context *detachedContext) *git {
-	env := new(MockedEnvironment)
-	env.On("isWsl", nil).Return(false)
-	env.On("hasFolder", "/rebase-merge").Return(context.rebaseMerge)
-	env.On("hasFolder", "/rebase-apply").Return(context.rebaseApply)
-	env.On("hasFolder", "/sequencer").Return(context.sequencer)
-	env.On("getFileContent", "/rebase-merge/head-name").Return(context.origin)
-	env.On("getFileContent", "/rebase-merge/onto").Return(context.onto)
-	env.On("getFileContent", "/rebase-merge/msgnum").Return(context.step)
-	env.On("getFileContent", "/rebase-apply/next").Return(context.step)
-	env.On("getFileContent", "/rebase-merge/end").Return(context.total)
-	env.On("getFileContent", "/rebase-apply/last").Return(context.total)
-	env.On("getFileContent", "/rebase-apply/head-name").Return(context.origin)
-	env.On("getFileContent", "/CHERRY_PICK_HEAD").Return(context.cherryPickSHA)
-	env.On("getFileContent", "/REVERT_HEAD").Return(context.revertSHA)
-	env.On("getFileContent", "/MERGE_MSG").Return(fmt.Sprintf("%s '%s' into %s", context.mergeMsgStart, context.mergeHEAD, context.onto))
-	env.On("getFileContent", "/sequencer/todo").Return(context.sequencerTodo)
-	env.On("getFileContent", "/HEAD").Return(context.branchName)
-	env.On("hasFilesInDir", "", "CHERRY_PICK_HEAD").Return(context.cherryPick)
-	env.On("hasFilesInDir", "", "REVERT_HEAD").Return(context.revert)
-	env.On("hasFilesInDir", "", "MERGE_MSG").Return(context.merge)
-	env.On("hasFilesInDir", "", "MERGE_HEAD").Return(context.merge)
-	env.On("hasFilesInDir", "", "sequencer/todo").Return(context.sequencer)
-	env.mockGitCommand(context.currentCommit, "rev-parse", "--short", "HEAD")
-	env.mockGitCommand(context.tagName, "describe", "--tags", "--exact-match")
-	env.mockGitCommand(context.origin, "name-rev", "--name-only", "--exclude=tags/*", context.origin)
-	env.mockGitCommand(context.onto, "name-rev", "--name-only", "--exclude=tags/*", context.onto)
-	env.mockGitCommand(context.branchName, "branch", "--show-current")
-	env.mockGitCommand(context.status, "status", "-unormal", "--short", "--branch")
-	env.On("getRuntimeGOOS", nil).Return("unix")
-	g := &git{
-		env:              env,
-		gitWorkingFolder: "",
-		Working:          &GitStatus{},
-		Staging:          &GitStatus{},
-	}
-	return g
-}
-
 func (m *MockedEnvironment) mockGitCommand(returnValue string, args ...string) {
 	args = append([]string{"-C", "", "--no-optional-locks", "-c", "core.quotepath=false", "-c", "color.status=false"}, args...)
 	m.On("runCommand", "git", args).Return(returnValue, nil)
 }
 
-func TestGetGitDetachedCommitHash(t *testing.T) {
-	want := "\uf417lalasha1"
-	context := &detachedContext{
-		currentCommit: "lalasha1",
+func TestSetGitHEADContextClean(t *testing.T) {
+	cases := []struct {
+		Case        string
+		Expected    string
+		Ref         string
+		RebaseMerge bool
+		RebaseApply bool
+		Merge       bool
+		CherryPick  bool
+		Revert      bool
+		Sequencer   bool
+		Ours        string
+		Theirs      string
+		Step        string
+		Total       string
+	}{
+		{Case: "detached on commit", Ref: DETACHED, Expected: "branch detached at commit 1234567"},
+		{Case: "not detached, clean", Ref: "main", Expected: "branch main"},
+		{
+			Case:        "rebase merge",
+			Ref:         DETACHED,
+			Expected:    "rebase branch origin/main onto branch main (1/2) at commit 1234567",
+			RebaseMerge: true,
+			Ours:        "refs/heads/origin/main",
+			Theirs:      "main",
+			Step:        "1",
+			Total:       "2",
+		},
+		{
+			Case:        "rebase apply",
+			Ref:         DETACHED,
+			Expected:    "rebase branch origin/main (1/2) at commit 1234567",
+			RebaseApply: true,
+			Ours:        "refs/heads/origin/main",
+			Step:        "1",
+			Total:       "2",
+		},
+		{
+			Case:     "merge branch",
+			Ref:      "main",
+			Expected: "merge branch feat-1 into branch main",
+			Merge:    true,
+			Theirs:   "branch 'feat-1'",
+			Ours:     "main",
+		},
+		{
+			Case:     "merge commit",
+			Ref:      "main",
+			Expected: "merge commit 1234567 into branch main",
+			Merge:    true,
+			Theirs:   "commit '123456789101112'",
+			Ours:     "main",
+		},
+		{
+			Case:     "merge tag",
+			Ref:      "main",
+			Expected: "merge tag 1.2.4 into branch main",
+			Merge:    true,
+			Theirs:   "tag '1.2.4'",
+			Ours:     "main",
+		},
+		{
+			Case:       "cherry pick",
+			Ref:        "main",
+			Expected:   "pick commit 1234567 onto branch main",
+			CherryPick: true,
+			Theirs:     "123456789101012",
+			Ours:       "main",
+		},
+		{
+			Case:     "revert",
+			Ref:      "main",
+			Expected: "revert commit 1234567 onto branch main",
+			Revert:   true,
+			Theirs:   "123456789101012",
+			Ours:     "main",
+		},
+		{
+			Case:      "sequencer cherry",
+			Ref:       "main",
+			Expected:  "pick commit 1234567 onto branch main",
+			Sequencer: true,
+			Theirs:    "pick 123456789101012",
+			Ours:      "main",
+		},
+		{
+			Case:      "sequencer cherry p",
+			Ref:       "main",
+			Expected:  "pick commit 1234567 onto branch main",
+			Sequencer: true,
+			Theirs:    "p 123456789101012",
+			Ours:      "main",
+		},
+		{
+			Case:      "sequencer revert",
+			Ref:       "main",
+			Expected:  "revert commit 1234567 onto branch main",
+			Sequencer: true,
+			Theirs:    "revert 123456789101012",
+			Ours:      "main",
+		},
+	}
+	for _, tc := range cases {
+		env := new(MockedEnvironment)
+		env.On("getRuntimeGOOS", nil).Return("unix")
+		env.On("isWsl", nil).Return(false)
+		env.mockGitCommand("", "describe", "--tags", "--exact-match")
+		env.mockGitCommand(tc.Theirs, "name-rev", "--name-only", "--exclude=tags/*", tc.Theirs)
+		env.mockGitCommand(tc.Ours, "name-rev", "--name-only", "--exclude=tags/*", tc.Ours)
+		// rebase merge
+		env.On("hasFolder", "/rebase-merge").Return(tc.RebaseMerge)
+		env.On("getFileContent", "/rebase-merge/head-name").Return(tc.Ours)
+		env.On("getFileContent", "/rebase-merge/onto").Return(tc.Theirs)
+		env.On("getFileContent", "/rebase-merge/msgnum").Return(tc.Step)
+		env.On("getFileContent", "/rebase-merge/end").Return(tc.Total)
+		// rebase apply
+		env.On("hasFolder", "/rebase-apply").Return(tc.RebaseApply)
+		env.On("getFileContent", "/rebase-apply/head-name").Return(tc.Ours)
+		env.On("getFileContent", "/rebase-apply/next").Return(tc.Step)
+		env.On("getFileContent", "/rebase-apply/last").Return(tc.Total)
+		// merge
+		env.On("hasFilesInDir", "", "MERGE_MSG").Return(tc.Merge)
+		env.On("getFileContent", "/MERGE_MSG").Return(fmt.Sprintf("Merge %s into %s", tc.Theirs, tc.Ours))
+		// cherry pick
+		env.On("hasFilesInDir", "", "CHERRY_PICK_HEAD").Return(tc.CherryPick)
+		env.On("getFileContent", "/CHERRY_PICK_HEAD").Return(tc.Theirs)
+		// revert
+		env.On("hasFilesInDir", "", "REVERT_HEAD").Return(tc.Revert)
+		env.On("getFileContent", "/REVERT_HEAD").Return(tc.Theirs)
+		// sequencer
+		env.On("hasFilesInDir", "", "sequencer/todo").Return(tc.Sequencer)
+		env.On("getFileContent", "/sequencer/todo").Return(tc.Theirs)
+
+		g := &git{
+			env: env,
+			props: map[Property]interface{}{
+				BranchIcon:     "branch ",
+				CommitIcon:     "commit ",
+				RebaseIcon:     "rebase ",
+				MergeIcon:      "merge ",
+				CherryPickIcon: "pick ",
+				TagIcon:        "tag ",
+				RevertIcon:     "revert ",
+			},
+			Hash: "1234567",
+			Ref:  tc.Ref,
+		}
+		g.setGitHEADContext()
+		assert.Equal(t, tc.Expected, g.HEAD, tc.Case)
 	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
 }
 
-func TestGetGitHEADContextTagName(t *testing.T) {
-	want := "\uf412lalasha1"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		tagName:       "lalasha1",
+func TestSetPrettyHEADName(t *testing.T) {
+	cases := []struct {
+		Case     string
+		Expected string
+		Hash     string
+		Tag      string
+		HEAD     string
+	}{
+		{Case: "main", Expected: "branch main", HEAD: BRANCHPREFIX + "main"},
+		{Case: "no hash", Expected: "commit 1234567", HEAD: "12345678910"},
+		{Case: "hash on tag", Hash: "132312322321", Expected: "tag tag-1", HEAD: "12345678910", Tag: "tag-1"},
+		{Case: "no hash on tag", Expected: "tag tag-1", Tag: "tag-1"},
+		{Case: "hash on commit", Hash: "1234567", Expected: "commit 1234567"},
+		{Case: "no hash on commit", Expected: "commit 1234567", HEAD: "12345678910"},
+	}
+	for _, tc := range cases {
+		env := new(MockedEnvironment)
+		env.On("getFileContent", "/HEAD").Return(tc.HEAD)
+		env.On("getRuntimeGOOS", nil).Return("unix")
+		env.On("isWsl", nil).Return(false)
+		env.mockGitCommand(tc.Tag, "describe", "--tags", "--exact-match")
+		g := &git{
+			env: env,
+			props: map[Property]interface{}{
+				BranchIcon: "branch ",
+				CommitIcon: "commit ",
+				TagIcon:    "tag ",
+			},
+			Hash: tc.Hash,
+		}
+		g.setPrettyHEADName()
+		assert.Equal(t, tc.Expected, g.HEAD, tc.Case)
 	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
 }
 
-func TestGetGitHEADContextRebaseMerge(t *testing.T) {
-	want := "\ue728 \ue0a0cool-feature-bro onto \ue0a0main (2/3) at \uf417whatever"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		rebase:        "true",
-		rebaseMerge:   true,
-		origin:        "cool-feature-bro",
-		onto:          "main",
-		step:          "2",
-		total:         "3",
+func TestSetGitStatus(t *testing.T) {
+	cases := []struct {
+		Case             string
+		Output           string
+		ExpectedWorking  *GitStatus
+		ExpectedStaging  *GitStatus
+		ExpectedHash     string
+		ExpectedRef      string
+		ExpectedUpstream string
+		ExpectedAhead    int
+		ExpectedBehind   int
+	}{
+		{
+			Case: "all different options on working and staging, no remote",
+			Output: `
+			# branch.oid 1234567891011121314
+			# branch.head rework-git-status
+			1 .R N...
+			1 .C N...
+			1 .M N...
+			1 .m N...
+			1 .? N...
+			1 .D N...
+			1 .A N...
+			1 .U N...
+			1 A. N...
+			`,
+			ExpectedWorking: &GitStatus{Modified: 4, Added: 2, Deleted: 1, Unmerged: 1},
+			ExpectedStaging: &GitStatus{Added: 1},
+			ExpectedHash:    "1234567",
+			ExpectedRef:     "rework-git-status",
+		},
+		{
+			Case: "all different options on working and staging, with remote",
+			Output: `
+			# branch.oid 1234567891011121314
+			# branch.head rework-git-status
+			# branch.upstream origin/rework-git-status
+			1 .R N...
+			1 .C N...
+			1 .M N...
+			1 .m N...
+			1 .? N...
+			1 .D N...
+			1 .A N...
+			1 .U N...
+			1 A. N...
+			`,
+			ExpectedWorking:  &GitStatus{Modified: 4, Added: 2, Deleted: 1, Unmerged: 1},
+			ExpectedStaging:  &GitStatus{Added: 1},
+			ExpectedUpstream: "origin/rework-git-status",
+			ExpectedHash:     "1234567",
+			ExpectedRef:      "rework-git-status",
+		},
+		{
+			Case: "remote with equal branch",
+			Output: `
+			# branch.oid 1234567891011121314
+			# branch.head rework-git-status
+			# branch.upstream origin/rework-git-status
+			`,
+			ExpectedUpstream: "origin/rework-git-status",
+			ExpectedHash:     "1234567",
+			ExpectedRef:      "rework-git-status",
+		},
+		{
+			Case: "remote with branch status",
+			Output: `
+			# branch.oid 1234567891011121314
+			# branch.head rework-git-status
+			# branch.upstream origin/rework-git-status
+			# branch.ab +2 -1
+			`,
+			ExpectedUpstream: "origin/rework-git-status",
+			ExpectedHash:     "1234567",
+			ExpectedRef:      "rework-git-status",
+			ExpectedAhead:    2,
+			ExpectedBehind:   1,
+		},
 	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextRebaseApply(t *testing.T) {
-	want := "\ue728 \ue0a0cool-feature-bro (2/3) at \uf417whatever"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		rebase:        "true",
-		rebaseApply:   true,
-		origin:        "cool-feature-bro",
-		step:          "2",
-		total:         "3",
+	for _, tc := range cases {
+		env := new(MockedEnvironment)
+		env.On("getRuntimeGOOS", nil).Return("unix")
+		env.On("isWsl", nil).Return(false)
+		env.mockGitCommand(strings.ReplaceAll(tc.Output, "\t", ""), "status", "-unormal", "--branch", "--porcelain=2")
+		g := &git{
+			env: env,
+		}
+		if tc.ExpectedWorking == nil {
+			tc.ExpectedWorking = &GitStatus{}
+		}
+		if tc.ExpectedStaging == nil {
+			tc.ExpectedStaging = &GitStatus{}
+		}
+		g.setGitStatus()
+		assert.Equal(t, tc.ExpectedStaging, g.Staging, tc.Case)
+		assert.Equal(t, tc.ExpectedWorking, g.Working, tc.Case)
+		assert.Equal(t, tc.ExpectedHash, g.Hash, tc.Case)
+		assert.Equal(t, tc.ExpectedRef, g.Ref, tc.Case)
+		assert.Equal(t, tc.ExpectedUpstream, g.Upstream, tc.Case)
+		assert.Equal(t, tc.ExpectedAhead, g.Ahead, tc.Case)
+		assert.Equal(t, tc.ExpectedBehind, g.Behind, tc.Case)
 	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextRebaseUnknown(t *testing.T) {
-	want := "\uf417whatever"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		rebase:        "true",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextCherryPickOnBranch(t *testing.T) {
-	want := "\ue29b pickme onto \ue0a0main"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		branchName:    "main",
-		cherryPick:    true,
-		cherryPickSHA: "pickme",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextCherryPickOnTag(t *testing.T) {
-	want := "\ue29b pickme onto \uf412v3.4.6"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		tagName:       "v3.4.6",
-		cherryPick:    true,
-		cherryPickSHA: "pickme",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextRevertOnBranch(t *testing.T) {
-	want := "\uf0e2 012345 onto \ue0a0main"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		branchName:    "main",
-		revert:        true,
-		revertSHA:     "01234567",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextRevertOnTag(t *testing.T) {
-	want := "\uf0e2 012345 onto \uf412v3.4.6"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		tagName:       "v3.4.6",
-		revert:        true,
-		revertSHA:     "01234567",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextSequencerCherryPickOnBranch(t *testing.T) {
-	want := "\ue29b pickme onto \ue0a0main"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		branchName:    "main",
-		sequencer:     true,
-		sequencerTodo: "pick pickme message\npick notme message",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextSequencerCherryPickOnTag(t *testing.T) {
-	want := "\ue29b pickme onto \uf412v3.4.6"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		tagName:       "v3.4.6",
-		sequencer:     true,
-		sequencerTodo: "pick pickme message\npick notme message",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextSequencerRevertOnBranch(t *testing.T) {
-	want := "\uf0e2 012345 onto \ue0a0main"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		branchName:    "main",
-		sequencer:     true,
-		sequencerTodo: "revert 01234567 message\nrevert notme message",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextSequencerRevertOnTag(t *testing.T) {
-	want := "\uf0e2 012345 onto \uf412v3.4.6"
-	context := &detachedContext{
-		currentCommit: "whatever",
-		tagName:       "v3.4.6",
-		sequencer:     true,
-		sequencerTodo: "revert 01234567 message\nrevert notme message",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextMerge(t *testing.T) {
-	want := "\ue727 \ue0a0feat into \ue0a0main"
-	context := &detachedContext{
-		merge:         true,
-		mergeHEAD:     "feat",
-		mergeMsgStart: "Merge branch",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextMergeRemote(t *testing.T) {
-	want := "\ue727 \ue0a0feat into \ue0a0main"
-	context := &detachedContext{
-		merge:         true,
-		mergeHEAD:     "feat",
-		mergeMsgStart: "Merge remote-tracking branch",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextMergeTag(t *testing.T) {
-	want := "\ue727 \uf412v7.8.9 into \ue0a0main"
-	context := &detachedContext{
-		merge:         true,
-		mergeHEAD:     "v7.8.9",
-		mergeMsgStart: "Merge tag",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextMergeCommit(t *testing.T) {
-	want := "\ue727 \uf4178d7e869 into \ue0a0main"
-	context := &detachedContext{
-		merge:         true,
-		mergeHEAD:     "8d7e869",
-		mergeMsgStart: "Merge commit",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("main")
-	assert.Equal(t, want, got)
-}
-
-func TestGetGitHEADContextMergeIntoTag(t *testing.T) {
-	want := "\ue727 \ue0a0feat into \uf412v3.4.6"
-	context := &detachedContext{
-		tagName:       "v3.4.6",
-		merge:         true,
-		mergeHEAD:     "feat",
-		mergeMsgStart: "Merge branch",
-	}
-	g := setupHEADContextEnv(context)
-	got := g.getGitHEADContext("")
-	assert.Equal(t, want, got)
 }
 
 func TestGetStashContextZeroEntries(t *testing.T) {
@@ -392,62 +396,6 @@ func TestGetStashContextZeroEntries(t *testing.T) {
 	}
 }
 
-func TestParseGitBranchInfoEqual(t *testing.T) {
-	g := git{}
-	branchInfo := "## master...origin/master"
-	got := g.parseGitStatusInfo(branchInfo)
-	assert.Equal(t, "master", got["local"])
-	assert.Equal(t, "origin/master", got["upstream"])
-	assert.Empty(t, got["ahead"])
-	assert.Empty(t, got["behind"])
-}
-
-func TestParseGitBranchInfoAhead(t *testing.T) {
-	g := git{}
-	branchInfo := "## master...origin/master [ahead 1]"
-	got := g.parseGitStatusInfo(branchInfo)
-	assert.Equal(t, "master", got["local"])
-	assert.Equal(t, "origin/master", got["upstream"])
-	assert.Equal(t, "1", got["ahead"])
-	assert.Empty(t, got["behind"])
-}
-
-func TestParseGitBranchInfoBehind(t *testing.T) {
-	g := git{}
-	branchInfo := "## master...origin/master [behind 1]"
-	got := g.parseGitStatusInfo(branchInfo)
-	assert.Equal(t, "master", got["local"])
-	assert.Equal(t, "origin/master", got["upstream"])
-	assert.Equal(t, "1", got["behind"])
-	assert.Empty(t, got["ahead"])
-}
-
-func TestParseGitBranchInfoBehindandAhead(t *testing.T) {
-	g := git{}
-	branchInfo := "## master...origin/master [ahead 1, behind 2]"
-	got := g.parseGitStatusInfo(branchInfo)
-	assert.Equal(t, "master", got["local"])
-	assert.Equal(t, "origin/master", got["upstream"])
-	assert.Equal(t, "2", got["behind"])
-	assert.Equal(t, "1", got["ahead"])
-}
-
-func TestParseGitBranchInfoNoRemote(t *testing.T) {
-	g := git{}
-	branchInfo := "## master"
-	got := g.parseGitStatusInfo(branchInfo)
-	assert.Equal(t, "master", got["local"])
-	assert.Empty(t, got["upstream"])
-}
-
-func TestParseGitBranchInfoRemoteGone(t *testing.T) {
-	g := git{}
-	branchInfo := "## test-branch...origin/test-branch [gone]"
-	got := g.parseGitStatusInfo(branchInfo)
-	assert.Equal(t, "test-branch", got["local"])
-	assert.Equal(t, "gone", got["upstream_status"])
-}
-
 func TestGitStatusUnmerged(t *testing.T) {
 	expected := "x1"
 	status := &GitStatus{
@@ -471,71 +419,6 @@ func TestGitStatusEmpty(t *testing.T) {
 	assert.Equal(t, expected, status.String())
 }
 
-func TestParseGitStatsWorking(t *testing.T) {
-	status := &GitStatus{}
-	output := []string{
-		"## amazing-feat",
-		" M change.go",
-		"DD change.go",
-		" ? change.go",
-		" ? change.go",
-		" A change.go",
-		" U change.go",
-		" R change.go",
-		" C change.go",
-	}
-	status.parse(output, true)
-	assert.Equal(t, 3, status.Modified)
-	assert.Equal(t, 1, status.Unmerged)
-	assert.Equal(t, 3, status.Added)
-	assert.Equal(t, 1, status.Deleted)
-	assert.True(t, status.Changed)
-}
-
-func TestParseGitStatsStaging(t *testing.T) {
-	status := &GitStatus{}
-	output := []string{
-		"## amazing-feat",
-		" M change.go",
-		"DD change.go",
-		" ? change.go",
-		"?? change.go",
-		" A change.go",
-		"DU change.go",
-		"MR change.go",
-		"AC change.go",
-	}
-	status.parse(output, false)
-	assert.Equal(t, 1, status.Modified)
-	assert.Equal(t, 0, status.Unmerged)
-	assert.Equal(t, 1, status.Added)
-	assert.Equal(t, 2, status.Deleted)
-	assert.True(t, status.Changed)
-}
-
-func TestParseGitStatsNoChanges(t *testing.T) {
-	status := &GitStatus{}
-	expected := &GitStatus{}
-	output := []string{
-		"## amazing-feat",
-	}
-	status.parse(output, false)
-	assert.Equal(t, expected, status)
-	assert.False(t, status.Changed)
-}
-
-func TestParseGitStatsInvalidLine(t *testing.T) {
-	status := &GitStatus{}
-	expected := &GitStatus{}
-	output := []string{
-		"## amazing-feat",
-		"#",
-	}
-	status.parse(output, false)
-	assert.Equal(t, expected, status)
-	assert.False(t, status.Changed)
-}
-
 func TestGitUpstream(t *testing.T) {
 	cases := []struct {
 		Case     string
@@ -580,7 +463,7 @@ func TestGetBranchStatus(t *testing.T) {
 		Behind   int
 		Upstream string
 	}{
-		{Case: "Equal with remote", Expected: " equal", Upstream: "main"},
+		{Case: "Equal with remote", Expected: " equal", Upstream: branchName},
 		{Case: "Ahead", Expected: " up2", Ahead: 2},
 		{Case: "Behind", Expected: " down8", Behind: 8},
 		{Case: "Behind and ahead", Expected: " up7 down8", Behind: 8, Ahead: 7},
@@ -601,7 +484,8 @@ func TestGetBranchStatus(t *testing.T) {
 			Behind:   tc.Behind,
 			Upstream: tc.Upstream,
 		}
-		assert.Equal(t, tc.Expected, g.getBranchStatus(), tc.Case)
+		g.setBranchStatus()
+		assert.Equal(t, tc.Expected, g.BranchStatus, tc.Case)
 	}
 }
 
@@ -657,7 +541,7 @@ func TestTruncateBranch(t *testing.T) {
 		g := &git{
 			props: props,
 		}
-		assert.Equal(t, tc.Expected, g.truncateBranch(tc.Branch), tc.Case)
+		assert.Equal(t, tc.Expected, g.formatHEAD(tc.Branch), tc.Case)
 	}
 }
 
@@ -684,7 +568,7 @@ func TestTruncateBranchWithSymbol(t *testing.T) {
 		g := &git{
 			props: props,
 		}
-		assert.Equal(t, tc.Expected, g.truncateBranch(tc.Branch), tc.Case)
+		assert.Equal(t, tc.Expected, g.formatHEAD(tc.Branch), tc.Case)
 	}
 }
 
@@ -730,10 +614,10 @@ func TestGitTemplateString(t *testing.T) {
 	}{
 		{
 			Case:     "Only HEAD name",
-			Expected: "main",
+			Expected: branchName,
 			Template: "{{ .HEAD }}",
 			Git: &git{
-				HEAD:   "main",
+				HEAD:   branchName,
 				Behind: 2,
 			},
 		},
@@ -742,23 +626,20 @@ func TestGitTemplateString(t *testing.T) {
 			Expected: "main \uF044 +2 ~3",
 			Template: "{{ .HEAD }}{{ if .Working.Changed }} \uF044 {{ .Working.String }}{{ end }}",
 			Git: &git{
-				HEAD: "main",
+				HEAD: branchName,
 				Working: &GitStatus{
 					Added:    2,
 					Modified: 3,
-					Changed:  true,
 				},
 			},
 		},
 		{
 			Case:     "No working area changes",
-			Expected: "main",
+			Expected: branchName,
 			Template: "{{ .HEAD }}{{ if .Working.Changed }} \uF044 {{ .Working.String }}{{ end }}",
 			Git: &git{
-				HEAD: "main",
-				Working: &GitStatus{
-					Changed: false,
-				},
+				HEAD:    branchName,
+				Working: &GitStatus{},
 			},
 		},
 		{
@@ -766,16 +647,14 @@ func TestGitTemplateString(t *testing.T) {
 			Expected: "main \uF046 +5 ~1 \uF044 +2 ~3",
 			Template: "{{ .HEAD }}{{ if .Staging.Changed }} \uF046 {{ .Staging.String }}{{ end }}{{ if .Working.Changed }} \uF044 {{ .Working.String }}{{ end }}",
 			Git: &git{
-				HEAD: "main",
+				HEAD: branchName,
 				Working: &GitStatus{
 					Added:    2,
 					Modified: 3,
-					Changed:  true,
 				},
 				Staging: &GitStatus{
 					Added:    5,
 					Modified: 1,
-					Changed:  true,
 				},
 			},
 		},
@@ -784,16 +663,14 @@ func TestGitTemplateString(t *testing.T) {
 			Expected: "main \uF046 +5 ~1 | \uF044 +2 ~3",
 			Template: "{{ .HEAD }}{{ if .Staging.Changed }} \uF046 {{ .Staging.String }}{{ end }}{{ if and (.Working.Changed) (.Staging.Changed) }} |{{ end }}{{ if .Working.Changed }} \uF044 {{ .Working.String }}{{ end }}", //nolint:lll
 			Git: &git{
-				HEAD: "main",
+				HEAD: branchName,
 				Working: &GitStatus{
 					Added:    2,
 					Modified: 3,
-					Changed:  true,
 				},
 				Staging: &GitStatus{
 					Added:    5,
 					Modified: 1,
-					Changed:  true,
 				},
 			},
 		},
@@ -802,26 +679,24 @@ func TestGitTemplateString(t *testing.T) {
 			Expected: "main \uF046 +5 ~1 | \uF044 +2 ~3 \uf692 3",
 			Template: "{{ .HEAD }}{{ if .Staging.Changed }} \uF046 {{ .Staging.String }}{{ end }}{{ if and (.Working.Changed) (.Staging.Changed) }} |{{ end }}{{ if .Working.Changed }} \uF044 {{ .Working.String }}{{ end }}{{ if gt .StashCount 0 }} \uF692 {{ .StashCount }}{{ end }}", //nolint:lll
 			Git: &git{
-				HEAD: "main",
+				HEAD: branchName,
 				Working: &GitStatus{
 					Added:    2,
 					Modified: 3,
-					Changed:  true,
 				},
 				Staging: &GitStatus{
 					Added:    5,
 					Modified: 1,
-					Changed:  true,
 				},
 				StashCount: 3,
 			},
 		},
 		{
 			Case:     "No local changes",
-			Expected: "main",
+			Expected: branchName,
 			Template: "{{ .HEAD }}{{ if .Staging.Changed }} \uF046{{ .Staging.String }}{{ end }}{{ if .Working.Changed }} \uF044{{ .Working.String }}{{ end }}",
 			Git: &git{
-				HEAD:    "main",
+				HEAD:    branchName,
 				Staging: &GitStatus{},
 				Working: &GitStatus{},
 			},
@@ -831,7 +706,7 @@ func TestGitTemplateString(t *testing.T) {
 			Expected: "from GitHub on main",
 			Template: "from {{ .UpstreamIcon }} on {{ .HEAD }}",
 			Git: &git{
-				HEAD:         "main",
+				HEAD:         branchName,
 				Staging:      &GitStatus{},
 				Working:      &GitStatus{},
 				UpstreamIcon: "GitHub",
