@@ -3,7 +3,7 @@ package main
 import "strings"
 
 type poshgit struct {
-	props     *properties
+	props     properties
 	env       environmentInfo
 	gitStatus string
 }
@@ -22,7 +22,7 @@ func (p *poshgit) string() string {
 	return p.gitStatus
 }
 
-func (p *poshgit) init(props *properties, env environmentInfo) {
+func (p *poshgit) init(props properties, env environmentInfo) {
 	p.props = props
 	p.env = env
 }
